# -*- coding: utf-8 -*-

from django.http import Http404, HttpResponseNotAllowed
from djclsview import View

__all__ = ['Resource']


METHOD_ACTION_MAP = {
    'singular': {
        'show': {
            'GET': 'show',
            'POST': 'create',
            'PUT': 'update',
            'DELETE': 'destroy',
        },
        'new': {'GET': 'new'},
        'edit': {'GET': 'edit'},
    },

    'plural': {
        'index': {
            'GET': 'index',
            'POST': 'create'
        },
        'show': {
            'GET': 'show',
            'PUT': 'update',
            'POST': 'update',
            'DELETE': 'destroy',
        },
        'new': {'GET': 'new'},
        'edit': {'GET': 'edit'},
    }
}


class Resource(View):

    def __init__(self, request, *args, **params):
        self.request = request
        self.args = args
        self.params = params
        self._called_yet = False

    def __call__(self):
        """Dispatch to an action based on HTTP method + URL."""

        if self._called_yet:
            return self
        self._called_yet = True
        method = self.request.POST.get('_method', self.request.method).upper()
        url_action = self.params.pop('action')
        mode = self.params.pop('mode', None)
        return self._route(method, url_action, mode=mode)()

    def _route(self, method, url_action, mode=None):

        """
        Resolve a method, URL action and (optionally) mode to a 0-ary callable.

        The default behaviour is to map the URL action and the method via the
        `METHOD_ACTION_MAP` to an action on this resource.
        """

        try:
            action_name = METHOD_ACTION_MAP[mode][url_action][method]
        except KeyError:
            # Assume the developer knows what he/she is doing.
            action_name = url_action

        if hasattr(self, action_name):
            return getattr(self, action_name)
        elif mode:
            return lambda: self._not_allowed(mode, url_action)
        else:
            return self._not_found

    def _not_allowed(self, mode, url_action):
        """Return a `HttpResponseNotAllowed` for this mode and URL action."""

        allowed_methods = []
        for meth, action_name in METHOD_ACTION_MAP[mode][url_action].items():
            if hasattr(self, action_name):
                allowed_methods.append(meth)
        return HttpResponseNotAllowed(allowed_methods)

    def _not_found(self):
        raise Http404

    def _format(self):
        """Return a mimetype shortcode, in case there's no Accept header."""

        return self.request.GET.get('format')



# -*- coding: utf-8 -*-

import re


def camel_to_underscore(camel_string):

    """
    Convert a CamelCase string to under_score.

    Examples:

        >>> camel_to_underscore('SplitAtTheBoundaries')
        'split_at_the_boundaries'

        >>> camel_to_underscore('XYZResource')
        'xyz_resource'

        >>> camel_to_underscore('ResourceXYZ')
        'resource_xyz'

        >>> camel_to_underscore('XYZ')
        'xyz'

    """

    return re.sub(
        r'(((?<=[a-z])[A-Z])|([A-Z](?![A-Z]|$)))', r'_\1',
        camel_string).lower().strip('_')


def resource_name(resource_cls):
    """Return the name of a given resource, stripping 'Resource' off the end."""

    from dagny import Resource

    if isinstance(resource_cls, Resource):
        resource_cls = resource_cls.__class__

    name = resource_cls.__name__
    if name.endswith('Resource'):
        return name[:-8]
    return name
